class pe_razor(
  $dbpassword = 'razor',
  $pe_tarball_base_url = "https://pm.puppetlabs.com/puppet-enterprise"
) {
  # Assert the platform being used is one validated for Tech Preview of Razor.
  $is_on_el = $operatingsystem ? {
    'RedHat' => true,
    'CentOS' => true,
    default  => false
  }

  $is_on_6 = $operatingsystemmajrelease ? {
    6       => true,
    default => false
  }

  if !($is_on_el and $is_on_6) {
    crit("Sorry, the tech preview of Razor is only available for Red Hat Enterprise Linux 6.")
  }

  # Configuration-like stuff.
  $packages    = "/opt/puppet/packages"

  # Determine the file extension. Pre-releases are .tar, releases are .tar.gz
  $file_extension = $pe_build ? {
    /^[^-]+-.+/ => "tar",
    default => "tar.gz",
  }

  # @todo danielp 2013-12-31: This listing will need to be adjusted when more
  # platforms are validated for use with Razor.
  $relarch = "el-6-x86_64"
  $build   = "puppet-enterprise-${pe_build}-${relarch}"
  $file    = "${build}.${file_extension}"
  $root    = "${build}/packages/${relarch}"
  $url     = "${pe_tarball_base_url}/${pe_build}/${file}"

  $dest = "${packages}/razor-repo"

  # Implementation
  file { $packages:
    ensure => directory, owner => root, group => root, mode => 0755
  } ->

  exec { "unpack the razor repo":
    # That should allow someone on a 1Mbit link to succeed at this, without
    # hanging *forever* if the network is screwed.  A long time, sure, but
    # not forever.
    provider => shell, timeout => 3600,
    command  => template('pe_razor/install-repo.sh.erb'),
    path     => "/usr/local/bin:/bin:/usr/bin",
    creates  => "${dest}/repodata/repomd.xml.asc"
  } ->

  yumrepo { "pe-razor-tech-preview":
    baseurl  => "file://${dest}",
    enabled  => 1,
    gpgcheck => 0
  } ->

  class { 'pe_postgresql::pe': } ->

  # create our database
  pe_postgresql::db { 'razor':
    user       => 'razor',
    password   => $dbpassword,
    grant      => 'all'
  } ->

  package { "pe-razor-server": ensure => latest } ->

  # Configuration
  file { "/etc/puppetlabs/razor/config.yaml":
    ensure  => file, owner => root, group => pe-razor, mode => 0640,
    content => template('pe_razor/config.yaml.erb')
  } ->

  file { "/etc/puppetlabs/razor/shiro.ini":
    ensure  => file, owner => root, group => pe-razor, mode => 0640,
    content => template('pe_razor/shiro.ini.erb')
  } ->

  file { "/opt/puppet/var/razor/repo":
    ensure => directory, owner => 'pe-razor', group => 'pe-razor', mode => 0750
  } ->

  exec { "migrate the razor database":
    # Allow up to 60 minutes for this to run.  It should be super-fast, right
    # now, but if we get large installations and a migration that needs to do
    # substantial work and can't be lazy-evaluated, this will help.
    provider => shell, timeout => 3600,
    command  => template('pe_razor/do-migrate.sh.erb'),
    unless   => template('pe_razor/do-check-migrate.sh.erb'),
    notify   => Exec["redeploy the razor application to torquebox"]
  } ->

  exec { "unpack the microkernel":
    provider => shell, timeout => 3600,
    command  => template('pe_razor/install-mk.sh.erb'),
    path     => "/usr/local/bin:/bin:/usr/bin",
    creates  => "/opt/puppet/var/razor/repo/microkernel/initrd0.img"
  } ->

  service { "pe-razor-server": ensure => running, enable => true } ->

  # Ensure that we have deployed the application.
  exec { "deploy the razor application to torquebox":
    provider => shell,
    command  => template('pe_razor/do-deploy.sh.erb'),
    creates  => "/opt/puppet/share/torquebox/jboss/standalone/deployments/razor-server-knob.yml"
  } ->

  # This one is to work around the fact that `creates` means we never deploy
  # twice.  This one should only trigger if the update to our
  # database happened.
  exec { "redeploy the razor application to torquebox":
    provider    => shell,
    command     => template('pe_razor/do-deploy.sh.erb'),
    refreshonly => true
  }
}
